﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.UltraWinListView;

namespace WLV_LoadItemsAsync
{
    public partial class Form1 : Form
    {
        private int activatedItems = 0;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            var colFileSize = this.ultraListView1.SubItemColumns.Add("FileSize");
            var colFileType = this.ultraListView1.SubItemColumns.Add("FileType");
            var colDateModified = this.ultraListView1.SubItemColumns.Add("DateModified");

            colFileSize.DataType = typeof(int);
            colFileSize.Format = "#,###,##0 KB";
            colFileSize.SubItemAppearance.TextHAlign = HAlign.Right;
            colFileType.DataType = typeof(string);
            colDateModified.DataType = typeof(DateTime);
            colFileType.Text = "Type";
            colDateModified.Text = "Date Modified";

            string shortDateFormat =
              System.Globalization.CultureInfo.CurrentCulture.DateTimeFormat.ShortDatePattern;
            string shortTimeFormat =
              System.Globalization.CultureInfo.CurrentCulture.DateTimeFormat.ShortTimePattern;
            colDateModified.Format = string.Format("{0} {1}", shortDateFormat, shortTimeFormat);

            this.ultraListView1.MainColumn.DataType = typeof(string);
            this.ultraListView1.MainColumn.Text = "Name";

            this.ultraListView1.View = UltraListViewStyle.Details;

            var filter = new CustomCreationFilter();
            ultraListView1.CreationFilter = filter;

            filter.ItemInitialized += filter_ItemInitialized;

        }

        void filter_ItemInitialized(object sender, ItemActivatedEventArgs e)
        {
            ultraLabel3.Text = "Initialized Items: "+ ++activatedItems;
        }

        private void GetDirectoryItems(string folderPath)
        {
            // You might want ot consider also clearing the chached items in the CreationFilter 
            // when opening a new folder/rebuilding the list
            ultraListView1.Items.Clear();
            ultraLabel2.Text = "Loaded items: 0";
            ultraLabel3.Text = "Initialized Items: 0";
            activatedItems = 0;

            DirectoryInfo cDriveInfo = new DirectoryInfo(folderPath);

            DirectoryInfo[] directories = cDriveInfo.GetDirectories();
            FileInfo[] files = cDriveInfo.GetFiles();

            for (int i = 0; i < directories.Length; i++)
            {
                DirectoryInfo directoryInfo = directories[i];

                UltraListViewItem item = this.ultraListView1.Items.Add(directoryInfo.FullName, directoryInfo.Name);
                item.Tag = directoryInfo;
            }

            for (int i = 0; i < files.Length; i++)
            {
                FileInfo fileInfo = files[i];

                UltraListViewItem item = this.ultraListView1.Items.Add(fileInfo.FullName, fileInfo.Name);
                item.Tag = fileInfo;
            }

            ultraLabel2.Text = "Loaded items: " + ultraListView1.Items.Count;
        }

        private void ultraButton1_Click(object sender, EventArgs e)
        {
            if (folderBrowserDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                GetDirectoryItems(folderBrowserDialog1.SelectedPath);
            }
        }
    }
}
